#ifndef lint
static char	*sccsid = "@(#)fakesyslog.c	1.3	(Berkeley) 2/6/88";
#endif

/*
 * Fake syslog routines for systems that don't have syslog.
 * Taken from an idea by Paul McKenny, <mckenny@sri-unix.arpa>.
 * (Unfortunately, Paul, I can't distribute the real syslog code
 * as you suggested ... sigh.)
 *
 * Warning: this file contains joe code that may offend you.
 */

#define FAKESYSLOG "/usr/local/lib/irc/logs/ircd.syslog"

#ifdef FAKESYSLOG

#include <time.h>
#include <stdio.h>
#include <sys/types.h>
#include <unistd.h>
#include <fcntl.h>
#include <syslog.h>

extern	char	*strcpy(), *strcat(), *ctime();
extern	int	errno;
extern	int	sys_nerr;
extern	char	*sys_errlist[];

static	int	logfd = -1;
static	u_long	mask = 0, opts = 0;
static	char	*tag = NULL, loghost[64];
static	FILE	*logfp = NULL;

static	char	*months[] = {"Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul",
			     "Aug", "Sep", "Oct", "Nov", "Dec"};

void	openlog(ident, logopt, facility)
int	logopt, facility;
char	*ident;
{
	if (logfd != -1)
		(void) close(logfd);

	logfd = open(FAKESYSLOG, O_APPEND|O_WRONLY, 0);

	if (logfd == -1)
		return;
	if (tag)
		(void) free(tag);
	tag = NULL;
	if (ident && *ident)
	    {
		tag = (char *)malloc(strlen(ident)+1);
		(void) strcpy(tag, ident);
	    }

	opts = logopt;

	if (logopt & LOG_NDELAY)
	    {
		int	t;

		t = fcntl(logfd, F_GETFL, 0);
		if (t != -1)
			(void) fcntl(logfd, F_SETFL, t|O_NDELAY);
	    }
	(void) gethostname(loghost, sizeof(loghost) - 1);
	logfp = fdopen(logfd, "a");
}


u_long	setlogmask(newmask)
u_long	newmask;
{
	u_long	old = (LOG_PRIMASK & mask);
	mask &= LOG_FACMASK;
	mask |= (LOG_PRIMASK & newmask);
	return old;
}


void	syslog(pri, msg, x1, x2, x3, x4, x5, x6, x7, x8, x9)
int	pri;
char	*msg, *x1, *x2, *x3, *x4, *x5, *x6, *x7, *x8, *x9;
{
	struct	tm	*tm;
	char	buf[1024], pid[8];
	char	*cp, *bp;
	long	clock;
	int	fd, err = errno;	/* time(2) may change it! */

	if ((fd = logfd) == -1)
		fd = open("/dev/console", O_APPEND|O_WRONLY, 0);
	if (fd == -1)
		return;
	(void) time(&clock);
	tm = localtime(clock);
	if (opts & LOG_PID)
		(void) sprintf(pid, "[%d]", getpid());
	else
		*pid = '\0';
	(void) sprintf(buf, "%s %d %d:%02d:%02d %s %s%s:",
			months[tm->tm_mon], tm->tm_mday, tm->tm_hour,
			tm->tm_min, tm->tm_sec, loghost, tag, pid);

	bp = buf + strlen(buf)-1;
	*bp++ = ' ';
	*bp = '\0';
	for (cp = msg; *cp; cp++) {
		if (*cp == '%' && cp[1] == 'm') {
			*bp = '\0';
			if (err >= sys_nerr || err < 0) {
				char	work[32];
				sprintf(work, "unknown error #%d", err);
				(void) strcat(bp, work);
			} else
				(void) strcat(bp, sys_errlist[err]);
			bp = buf + strlen(buf);
			cp++;
		} else {
			*bp++ = *cp;
		}
	}
	*bp = '\0';
	/* Ah, the semantic security of C ... */
	if (bp[-1] != '\n')
		(void) strcat(bp, "\n");

	(void) fprintf(logfp, buf, x1, x2, x3, x4, x5, x6, x7, x8, x9);
}

void	closelog()
{
	if (logfd != -1)
		(void) close(logfd);
	logfd = -1;
	if (logfp)
		(void) fclose(logfp);
	logfp = NULL;
	if (tag)
		(void) free(tag);
	tag = NULL;
}
#endif
